#  Roombot
#  CSC297 @ University of Rochester

import struct, os, sys
import appscript, cherrypy, serial, osax
import templates

QTBR_SETTINGS_FILE = "roomba.qtbr"
ROBOT_VOICE = "Zarvox"

class Controller(object):
    def __init__(self):
        self.sa = osax.OSAX();
    
        self.ser = serial.Serial()
        self.ser.port = raw_input("Serial port: ")
        self.ser.baudrate = 57600
        
        self.ser.close() #try to get around potential serial port lock
        self.ser.open()
        self.ser.write(self.opcode(128)) #start-up opcodes
        self.ser.write(self.opcode(132))

    def isBump(self):
        self.ser.write(self.opcode(142))

        self.ser.write(self.opcode(7))
        sensorbyte = struct.unpack("b", self.ser.read(1))[0]

        return sensorbyte & 1 or sensorbyte & 2 >> 1 #left or right bump

    def drive(self, motor1, motor2):
        self.ser.write(self.opcode(145)) #direct drive opcode

        self.ser.write(self.opcode((motor1 & 0xff00) >> 8)) #high byte mask
        self.ser.write(self.opcode(motor1 & 0x00ff)) #low byte mask

        self.ser.write(self.opcode((motor2 & 0xff00) >> 8))
        self.ser.write(self.opcode(motor2 & 0x00ff))

    def opcode(self, oc):
        return bytes(chr(oc))
        
    #start QuickTime Broadcaster with settings file with client's IP
    def generate_settings(self, ip):
        f = open(QTBR_SETTINGS_FILE, "w")
        f.write(templates.QTBR_SETTINGS.format(ip))
        f.close()
        qtb = appscript.app("QuickTime Broadcaster") 
        qtb.documents.stop()
        qtb.open(str(os.path.abspath(QTBR_SETTINGS_FILE))).start() #use appscript to launch qtb
        
class Server:
    def __init__(self):
        self.controller = Controller()
        
        port = raw_input("Server port [80]: ")
        if not port:
            port = 80
        else:
            port = int(port)
            
        cherrypy.config.update({
            "server.socket_port": port,
        })
    
    @cherrypy.expose
    def index(self, refresh_stream=True):
        if refresh_stream:
            self.controller.generate_settings(cherrypy.request.remote.ip) #relaunch qtb
        return templates.INDEX
        
    @cherrypy.expose
    def motors(self, motor1=0, motor2=0):
        motor1 = int(motor1)
        motor2 = int(motor2)
        if self.controller.isBump(): #if it's hitting something, turn until it's not
            motor1 = 100
            motor2 = -100
        self.controller.drive(motor1, motor2)
        return "success"
        
    @cherrypy.expose
    def message(self, text=""):
        print text
        self.controller.sa.say(text, using=ROBOT_VOICE) #text-to-speech
        return "success"
        
    @cherrypy.expose
    def stream(self):
        cherrypy.response.headers['content-type'] = "video/quicktime"
        return templates.STREAM.format(cherrypy.request.remote.ip) #return generated .mov with client's ip


if __name__ == '__main__':
    print os.path.abspath(os.path.dirname(sys.argv[0]))
    cherrypy.quickstart(Server(), config="roombot.conf") #launch cherrypy w/ roombot config
